%
%
%  Stereology SpaceBall for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions subdirectory of the Imaris
%    Installation directory (create this directory if necessary)
%  - Start Imaris
%  - You will now find this module as a button in surpass.
%
%  Do not edit the xml below:
%
%    <CustomTools>
%      <Menu>
%        <Item name="Stereology Space Ball" icon="Matlab" tooltip="Create a surface object with the shape of a half shsphere.">
%          <Command>MatlabXT::XTStereologySpaceBall(%i)</Command>
%        </Item>
%      </Menu>
%      <SurpassToolbar>
%        <Item name="Stereology Space Ball" tooltip="Create a surface object with the shape of a half sphere." icon="Matlab">
%          <Command>MatlabXT::XTStereologySpaceBall(%i)</Command>
%        </Item>
%      </SurpassToolbar>
%    </CustomTools>
%  
%
%  Description:
% 
%   Create a surface object with the shape of a half sphere. 
%   The radius size is defined by the user.
%
%

function XTStereologySpaceBall(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
  javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

if vImarisApplication.GetDataSet.GetSizeX == 0
  msgbox('Please Load a DataSet first');
  return;
end

vSpots = vImarisApplication.GetFactory.CreateSpots();
[vExtendsMin, vExtendsMax] = BPGetExtends_(vImarisApplication.GetDataSet);
vPosition = (vExtendsMin + vExtendsMax)/2;
vPosition(3) = vExtendsMax(3);
vRadius = min(vPosition-vExtendsMin);
vAnswer = inputdlg({'Radius of SpaceBall'}, 'Stereology Box', 1, {num2str(vRadius)});
if ~isempty(vAnswer)
  vRadius= str2double(vAnswer(1));
  
  %%
  vCreateSpot = false;
  if vCreateSpot
    vSizeT = vImarisApplication.GetDataSet.GetSizeT;
    vPositions = repmat(vPosition, [vSizeT, 1]);
    vNSpots = vSizeT;
    vRadii = repmat(vRadius, [vNSpots, 1]);
    vTimeIndices = floor((0:(vNSpots-1))/(vNSpots/vSizeT));
    vSpots.Set(vPositions, vTimeIndices, vRadii);
    vSpots.SetName('StereologySpaceBall');
    vImarisApplication.GetSurpassScene.AddChild(vSpots, -1);
  end

  vNTheta = 20;
  vVectors = CreateRadialVectors_(vNTheta);
  vVectors = vVectors(vVectors(:,3) <= 0,:);
  vNPhi = size(vVectors,1)/vNTheta;
  vVertices = vVectors * vRadius;
  vVertices(:,1) = vVertices(:,1) + vPosition(1);
  vVertices(:,2) = vVertices(:,2) + vPosition(2);
  vVertices(:,3) = vVertices(:,3) + vPosition(3);
  vNormals = vVectors;
  vTriangles = [[1,2,vNTheta+1]; [2,vNTheta+2,vNTheta+1]]; 
  vTriangles = repmat(vTriangles, [vNTheta-1,1]) + repmat(floor((0:(2*(vNTheta-1)-1))/2)',[1,3]);
  vTriangles = [vTriangles ; [vNTheta, 1, 2*vNTheta]; [1, vNTheta+1, 2*vNTheta]];
  vTriangles = repmat(vTriangles, [vNPhi-1,1]) + repmat(vNTheta*floor((0:(2*(vNTheta)*(vNPhi-1)-1))/(2*(vNTheta)))',[1,3]);
  if vImarisApplication.GetDataSet.GetSizeT > 1
    vGroup = vImarisApplication.GetFactory.CreateDataContainer();
    vImarisApplication.GetSurpassScene.AddChild(vGroup, -1);
    vGroup.SetName('StereologySpaceBall');
  else
    vGroup = vImarisApplication.GetSurpassScene;
  end
  for vTimeIndex = 0:(vImarisApplication.GetDataSet.GetSizeT-1)
    vSurfaceObject = vImarisApplication.GetFactory.CreateSurfaces();
    vSurfaceObject.AddSurface(vVertices, vTriangles-1, vNormals, vTimeIndex);
    vSurfaceObject.SetName(strcat('SpaceBall ', num2str(vTimeIndex)));
    vGroup.AddChild(vSurfaceObject, -1);
  end
end


%%
% function aVectors = CreateRadialVectors(vNSamplesEquator)
%
function aVectors = CreateRadialVectors_(vNSamplesEquator)

if nargin == 0
  vNSamplesEquator = 20;
end

vPhi = (0:vNSamplesEquator)/vNSamplesEquator*pi;

vCoord = []; 
vNTheta = vNSamplesEquator;
for vCount = 1:numel(vPhi); 
  vNewTheta = (1:vNTheta)/vNTheta*2*pi; 
  vNew = [repmat(vPhi(vCount),[vNTheta,1]),reshape(vNewTheta,[vNTheta,1])];
  vCoord = [vCoord; vNew]; 
end;

[aVectors(:,1), aVectors(:,2), aVectors(:,3)] = sph2cart(vCoord(:,2), vCoord(:,1)-pi/2, ones(size(vCoord,1),1));

%%
% function GetExtends
function [aExtendMin, aExtendMax] = BPGetExtends_(aImarisDataSet)

aExtendMin = [0,0,0];
aExtendMin(1)= aImarisDataSet.GetExtendMinX;
aExtendMin(2) = aImarisDataSet.GetExtendMinY;
aExtendMin(3) = aImarisDataSet.GetExtendMinZ;

aExtendMax = [0,0,0];
aExtendMax(1)= aImarisDataSet.GetExtendMaxX;
aExtendMax(2) = aImarisDataSet.GetExtendMaxY;
aExtendMax(3) = aImarisDataSet.GetExtendMaxZ;
